diff -Nur dovecot-cvs/src/auth/passdb-vpopmail.c dovecot/src/auth/passdb-vpopmail.c
--- dovecot-cvs/src/auth/passdb-vpopmail.c	2006-01-22 12:33:27.000000000 +0100
+++ dovecot/src/auth/passdb-vpopmail.c	2006-08-23 11:50:16.000000000 +0200
@@ -16,6 +16,88 @@
 
 #define VPOPMAIL_DEFAULT_PASS_SCHEME "CRYPT"
 
+struct passdb_vpopmail_request {
+	struct auth_request *auth_request;
+	union {
+	  verify_plain_callback_t *verify_plain;
+	  lookup_credentials_callback_t *lookup_credentials;
+	} callback;
+};
+
+
+static void vpopmail_lookup_pass(struct passdb_vpopmail_request *vpopmail_request)
+{
+	struct vqpasswd *vpw;
+	char vpop_user[VPOPMAIL_LIMIT], vpop_domain[VPOPMAIL_LIMIT];
+		  
+	struct auth_request *auth_request = vpopmail_request->auth_request;
+	
+	enum passdb_result passdb_result;
+	const char *user, *password, *scheme;
+	int ret;
+
+	passdb_result = PASSDB_RESULT_INTERNAL_FAILURE;
+	user = auth_request->user;
+	password = NULL;
+	
+	vpw = vpopmail_lookup_vqp(auth_request, vpop_user, vpop_domain);
+	if (vpw == NULL) {
+	    passdb_result = PASSDB_RESULT_USER_UNKNOWN;
+		vpopmail_request->callback.lookup_credentials(PASSDB_RESULT_USER_UNKNOWN, "", auth_request);
+		return;
+	}
+
+	if (((vpw->pw_gid & NO_IMAP) != 0 &&
+	     strcmp(auth_request->service, "IMAP") == 0) ||
+	    ((vpw->pw_gid & NO_POP) != 0 &&
+	     strcmp(auth_request->service, "POP3") == 0)) {
+		auth_request_log_info(auth_request, "vpopmail",
+				      "%s disabled", auth_request->service);
+	    passdb_result = PASSDB_RESULT_USER_UNKNOWN;
+		vpopmail_request->callback.lookup_credentials(PASSDB_RESULT_USER_DISABLED, "", auth_request);
+		return;
+	}
+	
+	passdb_result = PASSDB_RESULT_OK;
+
+	password = vpw->pw_clear_passwd;
+	scheme = "CLEARTEXT";
+
+	if (auth_request->credentials != -1) {
+		passdb_handle_credentials(passdb_result, password, scheme,
+			vpopmail_request->callback.lookup_credentials,
+			auth_request);
+		return;
+	}
+
+	/* verify plain */
+	if (password == NULL) {
+		vpopmail_request->callback.verify_plain(passdb_result, auth_request);
+		return;
+	}
+
+	ret = auth_request_password_verify(auth_request,
+					   auth_request->mech_password,
+					   password, scheme, "vpopmail");
+
+	vpopmail_request->callback.verify_plain(ret > 0 ? PASSDB_RESULT_OK :
+					   PASSDB_RESULT_PASSWORD_MISMATCH,
+					   auth_request);
+	auth_request_unref(&auth_request);
+}
+
+static void vpopmail_lookup_credentials(struct auth_request *request,
+					lookup_credentials_callback_t *callback)
+{
+	struct passdb_vpopmail_request *vpopmail_request;
+
+	vpopmail_request = p_new(request->pool, struct passdb_vpopmail_request, 1);
+	vpopmail_request->auth_request = request;
+	vpopmail_request->callback.lookup_credentials = callback;
+
+        vpopmail_lookup_pass(vpopmail_request);
+}
+
 static void
 vpopmail_verify_plain(struct auth_request *request, const char *password,
 		      verify_plain_callback_t *callback)
@@ -114,7 +196,7 @@
 	vpopmail_deinit,
 
 	vpopmail_verify_plain,
-	NULL
+	vpopmail_lookup_credentials
 };
 
 #endif
